#!/usr/bin/env python3
import time
import random
from pprint import pprint
from .agent_based_api.v1 import get_value_store, get_rate, register, render, Service, Result, State, Metric

def getStateUpper(levels, value):
    warn, crit = levels
    if value >= crit:
        return State.CRIT
    if value >= warn:
        return State.WARN
    return State.OK

# services with item --> storageid and usage of value_store (to be able to calculate rates)
def discover_hal9001_storages(section):
    for key in section:
        yield(Service(item = key))

def check_hal9001_storages(item, params, section):
    curr_time = time.time() # needed for calculating rates
    storageid = item
    # get all values from section
    realname = section[storageid][0]
    ul_bytes = section[storageid][1]
    dl_bytes = section[storageid][2]

    # these are the keys for our value_store
    vs_key_ul = f"hal_{storageid}_ul"
    vs_key_dl = f"hal_{storageid}_dl"
    vs_key_time = f"hal_{storageid}_time" # this is necessary only because we did not get get_rate() to work

    # we have to persist our upload/download values so that we can calculate rates
    value_store = get_value_store()
    # get the values from last run
    ul_last = value_store.get(vs_key_ul, 0)
    dl_last = value_store.get(vs_key_dl, 0)
    time_last = value_store.get(vs_key_time, 0)
    # store the new values
    value_store[vs_key_ul] = ul_bytes
    value_store[vs_key_dl] = dl_bytes
    value_store[vs_key_time] = curr_time

    # now calculate the rates
    # in theory this should work with calls to get_rate() - but unfortunately
    # these calls cause an error, why, how to call function get_rate() correctly?? TBD
    '''
    File "/omd/sites/ttw/lib/python3/cmk/base/api/agent_based/utils.py", line 494, in get_rate
    if not last_state or len(last_state) != 2:
    TypeError: object of type 'int' has no len()
    '''
    #ul_rate = get_rate(value_store, f"{storageid}_ul", curr_time, ul_bytes, raise_overflow=True)
    #dl_rate = get_rate(value_store, f"{storageid}_dl", curr_time, dl_bytes, raise_overflow=True)
      
    time_delta = curr_time - time_last
    # avoid dividing by null and getting negative rates
    if time_delta <= 0:
        ul_rate = 1024.0
        dl_rate = 2048.0
    else:
        ul_rate = (ul_bytes - ul_last)/time_delta
        dl_rate = (dl_bytes - dl_last)/time_delta

    # levels are defined as MBytes/s in our rule, so we have to adjust this here
    # because we are getting Bytes/s from our rate calculation
    warn, crit = params["levels_hal_storages_rates"]
    warn = warn*(1024**2)
    crit = crit*(1024**2)
    levels = (warn, crit)

    # create graphs for upload/download rates
    yield Metric("hal_storages_upload_rate", ul_rate, levels=levels)
    yield Metric("hal_storages_download_rate", dl_rate, levels=levels)

    # calculate state of service based on levels given as parameter
    state_ul = getStateUpper(levels, ul_rate)
    state_dl = getStateUpper(levels, dl_rate)
    summary_ul = f"Upload rate of '{realname}' is {render.iobandwidth(ul_rate)}"
    summary_dl = f"Download rate of '{realname}' is {render.iobandwidth(dl_rate)}"
    details_ul = f"Upload since last check: {render.bytes(ul_bytes)}"
    details_dl = f"Download since last check: {render.bytes(dl_bytes)}"

    # create services
    yield(Result(state=state_ul, summary=summary_ul, details=details_ul))
    yield(Result(state=state_dl, summary=summary_dl, details=details_dl))
 
def parse_hal9001_storages_section(string_table):
    # convert the raw output of the agent section in an meaningful structure, do type conversions and so on
    parsed_data = {}
    for line in string_table:
        storageid = line[0]
        realname = line[1]
        upload = int(float(line[2]))
        download = int(float(line[3]))
        parsed_data[f"{storageid}"] = [realname, upload, download]
    return parsed_data
      
register.agent_section(
    name = "hal9001_storages",
    parse_function = parse_hal9001_storages_section,
)

register.check_plugin(
    name = "hal9001_storages",
    service_name = "HAL Storage information %s",
    discovery_function = discover_hal9001_storages,
    check_function = check_hal9001_storages,
    check_default_parameters = {
        "levels_hal_storages_rates": (200.0, 300.0),
    },
    check_ruleset_name="hal9001_storages_storage_rates",
)